﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;

namespace EmulatorLCD
{
    [ToolboxItem(true)]
    public partial class EmulatorLCD : UserControl
    {
        public enum cursor_LCD
        {
            None,
            Line,
            Blink
        }

        public LcdData ddram {get; set; }

        private Timer _timer;
        private bool _blinked = false;
        private Bitmap _bmp = new Bitmap(400, 100);
        private Bitmap _dotfon = new Bitmap(3, 3);
        private Bitmap _dot = new Bitmap(3, 3);

        public EmulatorLCD()
        {
            SetStyle(ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint | ControlStyles.OptimizedDoubleBuffer | ControlStyles.ResizeRedraw, true);
            InitializeComponent();
            ddram = new LcdData(_rowCount, _charCount);
            _timer = new Timer();
            _timer.Tick += timer_Tick;
            _timer.Interval = _blink;
            _timer.Start();
            DrawFon();
            Draw_dot_and_dotfon();
        }
        private void DrawFon()
        {

            Width = 48 + _charCount * (1 + 3 * 5);
            Height = 51 + _rowCount * (1 + 3 * 8);
            _bmp = new Bitmap(Width, Height);
            Graphics _e = Graphics.FromImage(_bmp);
            _e.Clear(_fonColor);
            if (EmulatorLCD_resource.lcdbottom != null)
                for (int x = 0; x < Width; x++)
                    _e.DrawImageUnscaled(EmulatorLCD_resource.lcdbottom, x, Height - 21);

            if (EmulatorLCD_resource.lcdtop != null)
                for (int x = 0; x < Width; x++)
                    _e.DrawImageUnscaled(EmulatorLCD_resource.lcdtop, x, 0);

            if (EmulatorLCD_resource.lcdleft != null)
                for (int y = 0; y < Height; y++)
                    _e.DrawImageUnscaled(EmulatorLCD_resource.lcdleft, 0, y);

            if (EmulatorLCD_resource.lcdright != null)
                for (int y = 0; y < Height; y++)
                    _e.DrawImageUnscaled(EmulatorLCD_resource.lcdright, Width - 14, y);

            if (EmulatorLCD_resource.lcdlefttop != null)
                _e.DrawImageUnscaled(EmulatorLCD_resource.lcdlefttop, 0, 0);

            if (EmulatorLCD_resource.lcdleftbottom != null)
                _e.DrawImageUnscaled(EmulatorLCD_resource.lcdleftbottom, 0, Height - 37);

            if (EmulatorLCD_resource.lcdrightbottom != null)
                _e.DrawImageUnscaled(EmulatorLCD_resource.lcdrightbottom, Width - 16, Height - 37);

            if (EmulatorLCD_resource.lcdrighttop != null)
                _e.DrawImageUnscaled(EmulatorLCD_resource.lcdrighttop, Width - 16, 0);

            _e.Dispose();
        }
        private void Draw_dot_and_dotfon()
        {
            var _a = new Bitmap(2, 2);
            var _e = Graphics.FromImage(_a);
            _e.Clear(Color.FromArgb(_contr, _dotColor.R, _dotColor.G, _dotColor.B));

            _e = Graphics.FromImage(_dotfon);
            _e.Clear(_fonColor);
            _e.DrawImage(_a, 0, 0, 3, 3);

            _e = Graphics.FromImage(_a);
            _e.Clear(_dotColor);

            _e = Graphics.FromImage(_dot);
            _e.Clear(_fonColor);
            _e.DrawImage(_a, 0, 0, 3, 3);

            _a.Dispose();
            _e.Dispose();
        }

        private void EmulatorLCD_Paint(object sender, PaintEventArgs e)
        {
            int _left = Width - _bmp.Width;
            e.Graphics.DrawImageUnscaled(_bmp, _left, 0);
            int a = 1 + 3 * 5;
            int b = 1 + 3 * 8;
            for (int r = 0; r < _rowCount; r++)
                for (int c = 0; c < _charCount; c++)
                    for (int x = 0; x < 5; x++)
                        for (int y = 0; y < 8; y++)
                        {
                            bool d = ddram.dots[r, c][x, y];
                            if (c == _cursor_coordinate.X && r == _cursor_coordinate.Y)
                                if (_cursor != cursor_LCD.None)
                                    d = (_cursor == cursor_LCD.Blink && _blinked) ? true : y == 7;   
                            e.Graphics.DrawImageUnscaled(d ? _dot : _dotfon, _left + c * a + 3 * x + 25, r * b + 3 * y + 26);
                        }
        }

        private void timer_Tick (object sender, EventArgs e)
        {
           // if (this.DesignMode)
             //   return;
            _blinked = !_blinked;
            Invalidate();
        }

        #region Property
        Color _fonColor = Color.FromArgb(180, 210, 0);
        [DefaultValue(typeof(Color), "180; 210; 0"), Category("LCD"), Description("")]
        public Color BackgroundColor
        {
            get => _fonColor;
            set
            {
                _fonColor = value;
                Draw_dot_and_dotfon();
                DrawFon();
                Invalidate();
            }
        }
        Color _dotColor = Color.Black;
        [DefaultValue(typeof(Color), "Black"), Category("LCD"), Description("")]
        public Color DotsColor
        {
            get => _dotColor;
            set
            {
                _dotColor = value;
                Draw_dot_and_dotfon();
                Invalidate();
            }
        }

        byte _contr = 50;
        [DefaultValue(typeof(byte), "50"), Category("LCD"), Description("")]
        public byte DotsContrast
        {
            get => _contr;
            set
            {
                _contr = value;
                Draw_dot_and_dotfon();
                Invalidate();
            }
        }

        byte _charCount = 16;
        [DefaultValue(typeof(byte), "16"), Category("LCD"), Description("")]
        public byte CharInRowCount
        {
            get => _charCount;
            set
            {
                if (value > 0 && value < 41)
                    _charCount = value;
                ddram = new LcdData(_rowCount, _charCount);
                DrawFon();
                Invalidate();
            }
        }

        byte _rowCount = 2;
        [DefaultValue(typeof(byte), "2"), Category("LCD"), Description("")]
        public byte RowCount
        {
            get => _rowCount;
            set
            {
                if (value > 0 && value < 9)
                    _rowCount = value;
                ddram = new LcdData(_rowCount, _charCount);
                DrawFon();
                Invalidate();
            }
        }

        private cursor_LCD _cursor = cursor_LCD.None;
        [DefaultValue(typeof(cursor_LCD), "None"), Category("LCD"), Description("")]
        public cursor_LCD Cursor_LCD
        {
            get => _cursor;
            set
            {
                _cursor = value;
                Invalidate();
            }
        }
        private Point _cursor_coordinate = new Point(0, 0);
        [DefaultValue(typeof(Point), "0; 0"), Category("LCD"), Description("")]
        public Point Cursor_LCD_Coordinate
        {
            get => _cursor_coordinate;
            set
            {
                _cursor_coordinate = value;
                Invalidate();
            }
        }
        private int _blink = 500;
        [DefaultValue(typeof(int), "500"), Category("LCD"), Description("")]
        public int Blink_ms
        {
            get => _blink;
            set
            {
                _blink = value;
                _timer.Interval = _blink;
            }
        }
        #endregion
    }
}
